/*******************************

liburlauth public interface

*******************************/

#ifndef URLAUTH_H
#define URLAUTH_H

#ifdef __cplusplus
extern "C"{
#endif

#include <libwebguard-utils/wg_types.h>
#include "urlauthdata.h"

typedef struct auth_t *HAuth;


typedef struct auth_config
{
	wg_memcached_peer       memc_acl;                   /* [memcached_acl] section in .ini file */ /* used for permanent data: user accounts, acl, admin config */
	wg_memcached_peer       memc_tmp;                   /* [memcached_tmp] section in .ini file */ /* used for temporary data: http-sessions, audit records */
	str_t                   acl_group_key;
	str_t                   cred_group_key;
	str_t                   sessid_group_key;
	str_t                   user_ctx_group_key;
	str_t                   auth_blocker_group_key;
	str_t                   auth_counter_group_key;
	char                    *config_filename;
	time_t                  session_lifetime;           /* sec */ /* 0 - infinite lifetime*/
	int                     max_concurrent_sessions;    /* > 0 */
	int                     max_auth_attempts;
	time_t                  auth_blocker_lifetime;
	time_t                  auth_counter_lifetime;
	int                     is_black_list;              /* boolean */
	int                     login_case_insensitive;     /* boolean */
	int                     notice_period;              /*days*/ /* < 0 do not control */
	wg_range                username_len_range;
	wg_range                pswd_len_range;
} auth_config;


typedef struct auth_result_t
{
    auth_status_t   status;
    str_t           additional_info;
    str_t           func_name;
    /** TODO! 'reason' message should not be here, it should appear only
     * in ajax_response function */
    char*           reason;
    int             pass_expiration;

} auth_result_t;


static wg_inline void auth_result_init(auth_result_t* result)
{
    result->status = AUTH_ERROR;
    str_t_nullify(&result->additional_info);
    str_t_nullify(&result->func_name);
    result->reason = NULL;
    result->pass_expiration = -1;
}

static wg_inline void auth_result_clear(auth_result_t* result)
{
    if(result) {
        result->status = AUTH_ERROR;
        str_t_nullify(&result->additional_info);
        str_t_clear(&result->func_name);
        result->reason = NULL;
        result->pass_expiration = -1;
    }
}


/*******************************************************************************/
/*
 * инициализация полей струтуры auth_config
 */
static wg_inline void auth_init_config( auth_config* conf )
{
//set by memset():
//    conf->memc_acl = {NULL, 0};
//    conf->memc_tmp = {NULL, 0};
//    conf->acl_group_key = NULL;
//    conf->cred_group_key = NULL;
//    conf->sessid_group_key = NULL;
//    conf->user_ctx_group_key = NULL;
//    conf->auth_blocker_group_key = NULL;
//    conf->auth_counter_group_key = NULL;
//    conf->config_filename = NULL;
//    conf->session_lifetime = 0;
//    conf->max_concurrent_sessions = 0;
//    conf->max_auth_attempts = 0;
//    conf->auth_blocker_lifetime = 0;
//    conf->auth_counter_lifetime = 0;
//    conf->is_black_list = 0;
//    conf->login_case_insensitive = 0;
//    conf->notice_period = 0;
//    conf->username_len_range = wg_range_null;
//    conf->pswd_len_range = wg_range_null;

    memset( conf, 0, sizeof(auth_config) );
}


/*******************************************************************************/
/*
 * инициализировать структуру auth_config методом auth_init_config
 * освобождать память, выделенную под структуру auth_config, методом auth_clear_config()
 */
int auth_load_config(const char* filename,      // __in имя файла конфигурации
                        auth_config* conf);     // __out загруженная конфигурация

/*******************************************************************************/
/*
 * освобождение памяти, выделенной под поля структуры auth_config
 */
void auth_clear_config( auth_config* conf );

/*******************************************************************************/

HAuth auth_create(const auth_config* conf, int auth_id);	// __in конфигурация для создания auth_manager

/*******************************************************************************/

void auth_free(HAuth hauth);				// __in хэндл созданного auth_manager

/*******************************************************************************/

auth_status_t authorize_request(HAuth hauth,
                                const client_identity_t* identity,
                                const str_t* user_id,
                                const url_request_t* request,
                                auth_result_t* result);

/*******************************************************************************/

auth_status_t validate_credentials(HAuth hauth,
                                    const client_identity_t* identity,
                                    user_credentials* credentials,
                                    const url_request_t* request,
                                    auth_result_t* result);

/*******************************************************************************/

int logout_client(HAuth hauth, const client_identity_t* identity,
                    const str_t* user_id, const url_request_t* request);

/*******************************************************************************/

int create_session_for_user(HAuth hauth, const client_identity_t* identity,
                            const str_t* user_id, auth_result_t* result);

/*******************************************************************************/

int resolve_identity_to_user(HAuth hauth, const client_identity_t* identity,
                                str_t* user_id);

/*******************************************************************************/

int annul_session(HAuth hauth, const client_identity_t* identity);

/*******************************************************************************/

/* return: 0 - успех, -1 - ошибка */
int add_auth_audit_record( HAuth hauth, const audit_info* info );

#ifdef __cplusplus
}
#endif

#endif /* URLAUTH_H */
